/*
	ConvertDefaultsIntCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include <stdio.h>
#include "ConvertDefaultsIntCmd.h"
#include "Application.h"
#include <LineEdit.h>
#include <InfoDialogManager.h>

#include "OpResources.h"	// this is the only resources file with global resources


ConvertDefaultsIntCmd::ConvertDefaultsIntCmd ( const char *name, int active ) :  Cmd ( name, active )
{
	_popupRealised = False;
}


void ConvertDefaultsIntCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	char buf[256];
	char help[1024];
		
	_edit = new LineEdit ( ( char * ) name(), TRUE,
									&ConvertDefaultsIntCmd::enterCallback,
									&ConvertDefaultsIntCmd::cancelCallback,
									( void * ) this );	
                
	/* retrieve resource to be displayed */
 	snprintf ( buf, sizeof ( buf ), "%d,%d",mixwaveResources->convertOffsetInt(),
				mixwaveResources->convertScaleInt() );
	buf [ sizeof ( buf ) - 1 ] = 0;	
	_edit->setColumns ( strlen ( buf ) );	// set width before instantiation

	snprintf ( help, sizeof ( help ), "Specifies parameters for conversion to type \
integer: offset,scale; default 0,1" );
	help [ sizeof ( help ) - 1 ] = 0;
	_edit->setHelp ( help );
	_edit->execute();
	_edit->setText ( buf );	// can't set text before instantiation...
	_popupRealised = True;
}


void ConvertDefaultsIntCmd::undoCommand()
{
	// not implemented
}


void ConvertDefaultsIntCmd::enterCallback ( void *clientData, char *text )
{
	ConvertDefaultsIntCmd *obj = ( ConvertDefaultsIntCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->enter ( text );
		
	obj->_popupRealised = False;
	delete obj->_edit;
}

/* specify resource to be set on 'enter' or 'ok' */
void ConvertDefaultsIntCmd::enter ( char *text )
{
	// check for separator
	int sep = 0;
	
	for ( unsigned int count = 0; count < strlen ( text ); count++ )
		if ( count[text] == ',' )
			sep++;

	if ( sep < 1 )
	{
		char msg[300];

		snprintf ( msg, sizeof ( msg ), "%s invalid: missing comma separator.\n\
Format (integer): offset, scale", text );
		msg [ sizeof ( msg ) - 1 ] = 0;
		infoDialogManager->post ( msg );
		return;
	}

	int offset, scale;
	
	sscanf ( text, "%d,%d", &offset, &scale );
	mixwaveResources->convertOffsetInt ( offset );
	mixwaveResources->convertScaleInt ( scale );
}


void ConvertDefaultsIntCmd::cancelCallback ( void *clientData )
{
	ConvertDefaultsIntCmd *obj = ( ConvertDefaultsIntCmd * ) clientData;

	obj->_popupRealised = False;
	delete obj->_edit;
}
