/*
	HetroToAsciiCmd.cc

	Will Eighteen, 2006: will@opaline.demon.co.uk
*/

#include "HetroToAsciiCmd.h"
#include <FileAccess.h>
#include "wave/waveclass.h"
#include <MessageDialogManager.h>


HetroToAsciiCmd::HetroToAsciiCmd ( char *name, int active, char *txtFile, char *floatFile ) : BreakCmd ( name, active )
{
	strncpy ( _srcFile, txtFile, BUFSIZE );
	_srcFile [ sizeof ( _srcFile ) - 1 ] = 0;
	strncpy ( _dstFile, floatFile, BUFSIZE );
	_dstFile [ sizeof ( _dstFile ) - 1 ] = 0;
	_once = 0;
	_percentDone = 0;
	_error = 0;
	_intSize = sizeof ( short int );
	_trackNum = 1;
	_numElements = 1;
	updateTimeout ( 25 );
}


HetroToAsciiCmd::~HetroToAsciiCmd()
{	
	// empty
}
	

void HetroToAsciiCmd::doCommand()
{
	if ( _done == True )
		return;

	int percent;
	char text[5], buf[BUFSIZE];
	unsigned int backtrack, bufSize;
	int error = 0;

	bufSize = sizeof ( buf ) - 1;

	if ( !_once )
	{
		// can we open source for read?
		FileAccess *in = new FileAccess ( _srcFile, "r" );
		
		if ( in->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile read error: %s", _srcFile, in->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		// can we open dst for write?
		FileAccess *out = new FileAccess ( _dstFile, "w" );
		
		if ( out->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile write error: %s", _dstFile, out->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		if ( error )
		{
			delete in;
			delete out;
			_done = True;
			_error = error;
			return;
		}
		
		_in = in;
		_out = out;

		_in->readF ( &_num, _intSize, _numElements ); // 1st element is no. of tracks

		snprintf ( buf, bufSize, "; source file %s\n", _srcFile );
		_out->writeF ( buf );
		snprintf ( buf, bufSize, "; number of amp/frq track pairs:\n" );
		_out->writeF (buf );
		snprintf ( buf, bufSize, "%d\n", _num[0] );
		_out->writeF ( buf );
		snprintf ( buf, bufSize, ";\n" );
		_out->writeF ( buf );

		_once = 1;
	}

	if ( _in->filePos() < _in->fileSize() )
	{
		_in->readF ( &_num, _intSize, _numElements );

		switch ( _num[0] )
		{
			case -1:	/* begin amplitude breakpoint set */
				snprintf ( buf, bufSize, "; track %d\n", _trackNum );
				_out->writeF ( buf );
				_out->writeF ( "; amplitude breakpoint set\n" );
				_out->writeF ( "; time (ms), amplitude\n" );
				_numElements = 2;
				break;
			
			case -2:	/* begin frequency breakpoint set */
				snprintf ( buf, bufSize, "; track %d\n", _trackNum );
				_out->writeF ( buf );
				_out->writeF ( "; frequency breakpoint set\n" );
				_out->writeF ( "; time (ms), frequency (Hz.)\n" );
				_numElements = 2;
				break;
			
			case 32767:	/* track end */
				_out->writeF ( "; end track\n;\n" );
				_numElements = 1;
				_trackNum++;
				
				if ( _in->filePos() == _in->fileSize() )
					backtrack = 0;
				else
					backtrack = _intSize;

				_in->filePos ( _in->filePos() - backtrack );
				break;
		}

		if ( _numElements == 2 && _num[0] >= 0 )
		{
			snprintf ( buf, 12, "%d %d\n", _num[0], _num[1] );
			_out->writeF ( buf );
			buf[0] = 0;
		}

		percent = ( int ) ( ( float ) _in->filePos() / ( float ) _in->fileSize() * 100 );

		if ( _percentDone != percent )
		{
			_percentDone = percent;
			snprintf ( text, 5, "%d%%", _percentDone );
			updateMessage ( text );
		}
	}
	else
	{
		delete _in;
		delete _out;
		_done = True;
	}
}
