/*
	MixDefaultsCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include <stdio.h>
#include "MixDefaultsCmd.h"
#include "Application.h"
#include <LineEdit.h>
#include <InfoDialogManager.h>

#include "OpResources.h"	// this is the only resources file with global resources


MixDefaultsCmd::MixDefaultsCmd ( const char *name, int active ) :  Cmd ( name, active )
{
	_popupRealised = False;
}


void MixDefaultsCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	char buf[256];
	char help[320];
		
	_edit = new LineEdit ( ( char * ) name(), TRUE,
									&MixDefaultsCmd::enterCallback,
									&MixDefaultsCmd::cancelCallback, ( void * ) this );	
                
	/* retrieve resource to be displayed */
 	snprintf ( buf, sizeof ( buf ), "%f,%f",mixwaveResources->gain(),
				mixwaveResources->pan() );

	buf [ sizeof ( buf ) - 1 ] = 0;		
	_edit->setColumns ( strlen ( buf ) );	// set width before instantiation

	snprintf ( help, sizeof ( help ), "Specifies default mix parameters: gain, pan; \
default %f,%f", mixwaveResources->gain(), mixwaveResources->pan() );
				
	help [ sizeof ( help ) - 1 ] = 0;
	_edit->setHelp ( help );
	_edit->execute();
	_edit->setText ( buf );	// can't set text before instantiation...
	_popupRealised = True;
}


void MixDefaultsCmd::undoCommand()
{
	// not implemented
}


void MixDefaultsCmd::enterCallback ( void *clientData, char *text )
{
	MixDefaultsCmd *obj = ( MixDefaultsCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->enter ( text );

	obj->_popupRealised = False;
	delete obj->_edit;
}

/* specify resource to be set on 'enter' or 'ok' */
void MixDefaultsCmd::enter ( char *text )
{
	// check for separator
	int sep = 0;
	
	for ( unsigned int count = 0; count < strlen ( text ); count++ )
		if ( count[text] == ',' )
			sep++;

	if ( sep != 1 )
	{
		char msg[300];
		snprintf ( msg, sizeof ( msg ), "%s invalid: missing comma separator.\n\
Format: gain,pan", text );
		msg [ sizeof ( msg ) - 1 ] = 0;
		infoDialogManager->post ( msg );
		return;
	}

	float gain;
	float pan;
	
	sscanf ( text, "%f,%f", &gain, &pan );

	if ( gain < 0 )
	{
		char msg[50];
		snprintf ( msg, sizeof ( msg ), "gain %f invalid: gain must be positive.", gain );
		msg [ sizeof ( msg ) - 1 ] = 0;
		infoDialogManager->post ( msg );
		return;
	}
	
	if ( ( pan < 0 ) || ( pan > 1 ) )
	{
		char msg[50];
		snprintf ( msg, sizeof ( msg ), "pan %f invalid: 0 <= pan <= 1.", pan );
		msg [ sizeof ( msg ) - 1 ] = 0;
		infoDialogManager->post ( msg );
		return;
	}

	mixwaveResources->gain ( gain );
	mixwaveResources->pan ( pan );
}


void MixDefaultsCmd::cancelCallback ( void *clientData )
{
	MixDefaultsCmd *obj = ( MixDefaultsCmd * ) clientData;

	obj->_popupRealised = False;
	delete obj->_edit;
}
