/*
    ViewTemplateCmd.cc

    Will Eighteen, 2001: will@opaline.demon.co.uk
*/

#include <DMenuCmd.h>
#include <FileAccess.h>
#include "OpResources.h"
#include "ViewTemplateCmd.h"
#include "AppWindow.h"
#include "MessageDialogManager.h"


ViewTemplateCmd::ViewTemplateCmd ( const char *name, int active ) : NoUndoCmd ( name, active )
{
	_currentSamples = 0;
	_viewMenuRealised = False;
}


void ViewTemplateCmd::doCommand()
{
	unsigned int n;
	unsigned int maxNameLength = 0;
	
	if ( templatePak->Count() == 0 )
		return;
		
	if ( _viewMenuRealised == True )
		return;

	_arrayItems = templatePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*templatePak)[n].name ) > maxNameLength )
			maxNameLength = strlen ( (*templatePak)[n].name );

	n = 0;
	maxNameLength++;
	
	if ( maxNameLength < 5 )
		maxNameLength = 5;	// accomodate 'quit' at a minimum
		
	_arrayItems++;
	
	char strings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 1 ); n++ )
		strcpy ( (char *) &strings [n], (*templatePak)[n].name );

	strcpy ( strings [ n ], "Quit" );

	_dMenu = new DMenuCmd ( "Template view menu", TRUE, FALSE,
								&ViewTemplateCmd::dMenuSelectionCallback,
								( void * ) this, "Current template files",
								_arrayItems, maxNameLength, ( char * ) &strings );

	_viewMenuRealised = True;
	_dMenu->execute();
}


void ViewTemplateCmd::dMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	ViewTemplateCmd *obj = ( ViewTemplateCmd * ) clientData;

	// Since the menu is static with reference to the name strings
	// (quitOnSelect is false), the reference may be invalid; check by name.

	if ( strcmp ( label, "Quit" ) == 0 )
	{
		obj->_viewMenuRealised = False;			// menu will be popped down
		delete obj->_dMenu;
	}
	else
	{
		int count, found;
		
		// scan base list for returned name
		for ( count = 0; count < templatePak->Count(); count++ )
		{
			found = 0;
			
			if ( strcmp ( label, (*templatePak)[count].name ) == 0 )
			{
				if ( (*templatePak)[count].fLock == ACTIVE )
				{
					char msg[BUFSIZE];
					snprintf ( msg, sizeof ( msg ), "%s\nFile in use!", (*templatePak)[count].name );
					msg [ sizeof ( msg ) - 1 ] = 0;
					messageDialogManager->post ( msg );
					return;
				}

				obj->viewTemplate ( (*templatePak)[count].templateNum );
				found = 1;
				count = templatePak->Count();
			}
		}
			
		if ( !found && mixwaveResources->enableMessagePopups() == True )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "Error: %s\nFile not loaded.", label );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}
}


void ViewTemplateCmd::viewTemplate ( unsigned int selectedItemNum )
{
	if ( selectedItemNum == 0 )	// WM 'X' activated in menu popup
	{
		_viewMenuRealised = False;
		return;
	}

	FileAccess *in = new FileAccess ( (*templatePak)[selectedItemNum-1].name, "r+" );

	if ( in->errNum() != 0  )
	{
		char msg[BUFSIZE];

		snprintf ( msg, sizeof ( msg ), "%s\nError: %s", (*templatePak)[selectedItemNum-1].name,
					in->error() );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );
		delete in;
		return;
	}

	for ( int count = 0; count < templatePak->Count(); count++ )
		if ( (*templatePak)[count].graphNum == ( int ) selectedItemNum )
			return;	// wave editor is already created for this wave

	if ( (*templatePak)[selectedItemNum-1].dataSize == 0 )
	{
		char buf[BUFSIZE];
		snprintf ( buf, sizeof ( buf ), "%s\n No data in file (use 'Edit/Copy' to insert data).",
					(*templatePak)[selectedItemNum-1].name );

		buf [ sizeof ( buf ) - 1 ] = 0;
		messageDialogManager->post ( buf );
		_viewMenuRealised = False;
		delete in;
		return;
	}

	if ( (*templatePak)[selectedItemNum-1].dataType == 0 )
	{
		if ( (*templatePak)[selectedItemNum-1].intArraySize > (*templatePak)[selectedItemNum-1].dataSize )
			(*templatePak)[selectedItemNum-1].intArraySize = (*templatePak)[selectedItemNum-1].dataSize;

		(*templatePak)[selectedItemNum-1].filePos = (*templatePak)[selectedItemNum-1].dataOffset * sizeof ( int );
		createDataArray ( selectedItemNum, (*templatePak)[selectedItemNum-1].intArraySize * sizeof ( int ) );
	}
	else
	{
		if ( (*templatePak)[selectedItemNum-1].fltArraySize > (*templatePak)[selectedItemNum-1].dataSize )
			(*templatePak)[selectedItemNum-1].fltArraySize = (*templatePak)[selectedItemNum-1].dataSize;

		(*templatePak)[selectedItemNum-1].filePos = (*templatePak)[selectedItemNum-1].dataOffset * sizeof ( float );
		createDataArray ( selectedItemNum, (*templatePak)[selectedItemNum-1].fltArraySize * sizeof ( float ) );
	}

	delete in;

	GraphIntCmd *graphIntCmd;
	GraphFltCmd *graphFltCmd;
	int width, height;

	sscanf ( mixwaveResources->viewWindowSize(), "%dx%d", &width, &height );

	if ( (*templatePak)[selectedItemNum-1].dataType == 0 )
	{
		graphIntCmd = new GraphIntCmd ( (*templatePak)[selectedItemNum-1].name,
										TRUE, &ViewTemplateCmd::viewTemplateIntCmdCallback,
										mixwaveResources->numDigits(),
										(*templatePak)[selectedItemNum-1].pIntArray,
										(*templatePak)[selectedItemNum-1].dataOffset,
										_currentSamples,
										(*templatePak)[selectedItemNum-1].dataSize,
										(*templatePak)[selectedItemNum-1].templateNum,
										( void * ) this );

		(*templatePak)[selectedItemNum-1].graphIntCmd = graphIntCmd;
		graphIntCmd->setWindowSize ( width, height );
		graphIntCmd->execute();
		graphIntCmd->setSelection ( (*templatePak)[selectedItemNum-1].startSel, (*templatePak)[selectedItemNum-1].endSel );
		graphIntCmd->setDatumValueBounds ( ( int ) (*templatePak)[selectedItemNum-1].min, ( int ) (*templatePak)[selectedItemNum-1].max );

		if ( (*templatePak)[selectedItemNum-1].min < 0 )
			graphIntCmd->centreZero ( True );	// set to signed data type
		else
			graphIntCmd->centreZero ( False );
	}
	else
	{
		graphFltCmd = new GraphFltCmd ( (*templatePak)[selectedItemNum-1].name,
										TRUE, &ViewTemplateCmd::viewTemplateFltCmdCallback,
										mixwaveResources->numDigits(),
										(*templatePak)[selectedItemNum-1].pFloatArray,
										(*templatePak)[selectedItemNum-1].dataOffset,
										_currentSamples,
										(*templatePak)[selectedItemNum-1].dataSize,
										(*templatePak)[selectedItemNum-1].templateNum,
										( void * ) this );

		(*templatePak)[selectedItemNum-1].graphFltCmd = graphFltCmd;
		graphFltCmd->setWindowSize ( width, height );
		graphFltCmd->execute();
		graphFltCmd->setSelection ( (*templatePak)[selectedItemNum-1].startSel, (*templatePak)[selectedItemNum-1].endSel );
		graphFltCmd->setDatumValueBounds ( (*templatePak)[selectedItemNum-1].min, (*templatePak)[selectedItemNum-1].max );

		if ( (*templatePak)[selectedItemNum-1].min < 0 )	// set by TemplateDefaultsCmd bitsPerSample
			graphFltCmd->centreZero ( True );	// set to signed data type
		else
			graphFltCmd->centreZero ( False );
	}

	// save the current view ID
	(*templatePak)[selectedItemNum-1].graphNum = (*templatePak)[selectedItemNum-1].templateNum;
}


void ViewTemplateCmd::viewTemplateIntCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, long yCurrent, Boolean reload,
								unsigned waveNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	ViewTemplateCmd *obj = ( ViewTemplateCmd * ) clientData;
	GraphIntCmd *gic = ( GraphIntCmd * ) cmd;

	if ( (*templatePak)[waveNum-1].fLock == INACTIVE )
		obj->viewTemplateCmdCallbackFinished ( gic, NULL, start, end, selStart, selEnd,
											xCurrent, yCurrent, 0,
											reload, waveNum, callbackType );
}


void ViewTemplateCmd::viewTemplateFltCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	ViewTemplateCmd *obj = ( ViewTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	if ( (*templatePak)[templateNum-1].fLock == INACTIVE )
		obj->viewTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
											xCurrent, 0, yCurrent,
											reload, templateNum, callbackType );
}


void ViewTemplateCmd::viewTemplateCmdCallbackFinished ( GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
								long selStart, long selEnd, long xCurrent, long
								yIntCurrent, float yFltCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
// yCurrent is duplicated so we can separate float from int here according to dataType
// debug - if xCurrent isn't valid for the array, yCurrent causes a seg fault
// printf("VTCCF %ld %ld %ld %ld %ld %ld %f\n", start, end, selStart, selEnd,xCurrent, yIntCurrent,yFltCurrent);
	if ( (*templatePak)[templateNum-1].fLock == ACTIVE )
	{
		if ( (*templatePak)[templateNum-1].dataType == 0 )	// integer
			(*templatePak)[templateNum-1].graphIntCmd->redisplay (
												(*templatePak)[templateNum-1].startRange,
												(*templatePak)[templateNum-1].endRange,
												(*templatePak)[templateNum-1].startSel,
												(*templatePak)[templateNum-1].endSel,
												(*templatePak)[templateNum-1].xCurrent,
												(*templatePak)[templateNum-1].dataOffset );
		else
			(*templatePak)[templateNum-1].graphFltCmd->redisplay (
												(*templatePak)[templateNum-1].startRange,
												(*templatePak)[templateNum-1].endRange,
												(*templatePak)[templateNum-1].startSel,
												(*templatePak)[templateNum-1].endSel,
												(*templatePak)[templateNum-1].xCurrent,
												(*templatePak)[templateNum-1].dataOffset );
		
		return;
	}

	int dataChanged = 0;

	if ( (*templatePak)[templateNum-1].dataType == 0 )
		dataChanged += (*templatePak)[templateNum-1].graphIntCmd->dataChanged();
	else
		dataChanged += (*templatePak)[templateNum-1].graphFltCmd->dataChanged();

	dataChanged += (*templatePak)[templateNum-1].templateChanged;	// if set externally
	(*templatePak)[templateNum-1].templateChanged = 0;
	
	if ( dataChanged != 0 )
	{
		int offset;

		if ( (*templatePak)[templateNum-1].dataType == 0 )
		{
			offset = (*templatePak)[templateNum-1].graphIntCmd->startData();
			offset *= sizeof ( int );
		}
		else
		{
			offset = (*templatePak)[templateNum-1].graphFltCmd->startData();
			offset *= sizeof ( float );
		}

		// since file length is unaltered, just contents, write the block out
		if ( (*templatePak)[templateNum-1].fLock == INACTIVE )
		{
			(*templatePak)[templateNum-1].fLock = ACTIVE;
			FileAccess *out = new FileAccess ( (*templatePak)[templateNum-1].name, "r+" );
			out->filePos ( offset );

			if ( (*templatePak)[templateNum-1].dataType == 0 )
				out->writeF ( (*templatePak)[templateNum-1].pIntArray, sizeof ( int ), (*templatePak)[templateNum-1].intArraySize );
			else
				out->writeF ( (*templatePak)[templateNum-1].pFloatArray, sizeof ( float ), (*templatePak)[templateNum-1].fltArraySize );

			delete out;
			(*templatePak)[templateNum-1].fLock = INACTIVE;			
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize, max, min;

	(*templatePak)[templateNum-1].startRange = start;
	(*templatePak)[templateNum-1].endRange = end;
	(*templatePak)[templateNum-1].startSel = selStart;
	(*templatePak)[templateNum-1].endSel = selEnd;
	(*templatePak)[templateNum-1].xCurrent = xCurrent;
	(*templatePak)[templateNum-1].yIntCurrent = yIntCurrent;
	(*templatePak)[templateNum-1].yFltCurrent = yFltCurrent;

	if ( callbackType != 0 )	// re-enable view popup on 'ok' or 'cancel'
	{
		if ( (*templatePak)[templateNum-1].dataType == 0 )
			(*templatePak)[templateNum-1].intArraySize = end - start + 1;
		else
			(*templatePak)[templateNum-1].fltArraySize = end - start + 1;
	
		(*templatePak)[templateNum-1].graphNum = 0;
		(*templatePak)[templateNum-1].dataOffset = (*templatePak)[templateNum-1].startRange;

		if ( (*templatePak)[templateNum-1].dataType == 0 )
		{
			delete (*templatePak)[templateNum-1].graphIntCmd;
			(*templatePak)[templateNum-1].graphIntCmd = NULL;
		}
		else
		{
			delete (*templatePak)[templateNum-1].graphFltCmd;
			(*templatePak)[templateNum-1].graphFltCmd = NULL;
		}
	}

	if ( callbackType == 2 )	// clear range and selection on 'Cancel'
	{
		(*templatePak)[templateNum-1].startRange = 0;
		(*templatePak)[templateNum-1].endRange = 0;
		(*templatePak)[templateNum-1].startSel = 0;
		(*templatePak)[templateNum-1].endSel = 0;
		(*templatePak)[templateNum-1].xCurrent = 0;
		(*templatePak)[templateNum-1].yIntCurrent = 0;
		(*templatePak)[templateNum-1].yFltCurrent = 0.0;
		(*templatePak)[templateNum-1].dataOffset = 0;
		return;
	}
	else
		if ( callbackType == 1 )
			return;	// 'Ok' - just keep pointers to array selection and quit

	if ( reload == True )
	{
		(*templatePak)[templateNum-1].templateChanged = False;

		min = start;

		if ( end < min )
			min = end;
		
		if ( selStart != selEnd )
		{
			if ( selStart < min )
				min = selStart;
	
			if ( selEnd < min )
				min = selEnd;
		}
		
		max = start;
		
		if ( end > max )
			max = end;

		if ( selStart != selEnd )
		{
			if ( selStart > max )
				max = selStart;
	
			if ( selEnd > max )
				max = selEnd;
		}

		if ( (*templatePak)[templateNum-1].dataType == 0 )
		{
			(*templatePak)[templateNum-1].filePos = min * sizeof ( int );
			blockSize = ( max - min + 1 ) * sizeof ( int );
		}
		else
		{
			(*templatePak)[templateNum-1].filePos = min * sizeof ( float );
			blockSize = ( max - min + 1 ) * sizeof ( float );
		}

		createDataArray ( templateNum, blockSize );

		if ( (*templatePak)[templateNum-1].dataType == 0 )
			gic->setDataSource ( (*templatePak)[templateNum-1].pIntArray, max - min + 1, (*templatePak)[templateNum-1].dataSize );
		else
			gfc->setDataSource ( (*templatePak)[templateNum-1].pFloatArray, max - min + 1, (*templatePak)[templateNum-1].dataSize );
	}

	if ( mixwaveResources->syncTemplateLength() == True )
	{	// match this template to its wave
		for ( int count = 0; count < wavePak->Count(); count++ )
		{
			if ( (*wavePak)[count].templateNum == templateNum )
			{
				if ( mixwaveResources->linkViews() == True )	// template drives wave
				{
					float ratio = (*templatePak)[templateNum-1].syncRatio;

					(*wavePak)[count].startRange = ( long ) ( ( float ) start * ratio );
					(*wavePak)[count].endRange = ( long ) ( ( float ) end * ratio );
					(*wavePak)[count].startSel = ( long ) ( ( float ) selStart * ratio );
					(*wavePak)[count].endSel = ( long ) ( ( float ) selEnd * ratio );
					(*wavePak)[count].xCurrent = ( long ) ( ( float ) xCurrent * ratio );
					(*wavePak)[count].dataOffset =  ( long ) ( ( float ) start * ratio );
					
					if ( (*wavePak)[count].graphCmd != NULL )
					{
						(*wavePak)[count].graphCmd->redisplay (
												(*wavePak)[count].startRange,
												(*wavePak)[count].endRange,
												(*wavePak)[count].startSel,
												(*wavePak)[count].endSel,
												(*wavePak)[count].xCurrent,
												(*wavePak)[count].dataOffset );
					}
				}
			}
		}
	}
}


void ViewTemplateCmd::createDataArray ( unsigned templateNum, long blockSize )
{
	if ( (*templatePak)[templateNum-1].dataType == 0 )
	{
		delete[] ( *templatePak )[templateNum-1].pIntArray;	// delete original array pointer
		_currentSamples = ( int ) blockSize / sizeof ( int );
		(*templatePak)[templateNum-1].pIntArray = new int [_currentSamples];
		(*templatePak)[templateNum-1].intArraySize = _currentSamples;
	}
	else
	{
		delete[] ( *templatePak )[templateNum-1].pFloatArray;	// delete original array pointer
		_currentSamples = ( int ) blockSize / sizeof ( float );
		(*templatePak)[templateNum-1].pFloatArray = new float [_currentSamples];
		(*templatePak)[templateNum-1].fltArraySize = _currentSamples;
	}

	if ( (*templatePak)[templateNum-1].fLock == INACTIVE )
	{
		(*templatePak)[templateNum-1].fLock = ACTIVE;
		FileAccess *in = new FileAccess ( (*templatePak)[templateNum-1].name, "r+" );

		in->numElements ( _currentSamples );

		if ( (*templatePak)[templateNum-1].dataType == 0 )
			in->elementSize ( sizeof ( int ) );	// int
		else
			in->elementSize ( sizeof ( float ) );	// int

		in->filePos ( (*templatePak)[templateNum-1].filePos );
			
		if ( (*templatePak)[templateNum-1].dataType == 0 )
			in->readF ( (*templatePak)[templateNum-1].pIntArray );
		else
			in->readF ( (*templatePak)[templateNum-1].pFloatArray );

		(*templatePak)[templateNum-1].fLock = INACTIVE;
		delete in;
	}
}
