// InterruptibleCmd.h - support user-interruptible activities

#ifndef BREAKCMD_H
#define BREAKCMD_h

#include <Xm/Xm.h>
#include "NoUndoCmd.h"


// define type for callback invoked when task is completed
typedef void ( *TaskDoneCallback ) ( class BreakCmd *, Boolean, void * );


class BreakCmd : public NoUndoCmd
{
	public:
		BreakCmd ( const char *, int active = TRUE );
		virtual ~BreakCmd();
		virtual void execute();	// override base class member function
		virtual void execute ( TaskDoneCallback, void * );
		
	protected:
		Boolean _done;
		virtual void cleanup();
		virtual void updateMessage ( char * );
		virtual void updateTimeout ( unsigned long );

	private:
		XtWorkProcId _wpId;
		TaskDoneCallback _callback;	// application-defined callback
		void *_clientData;			// provide by application
		Boolean workProc();
		static Boolean workProcCallback ( XtPointer );
		static void interruptCallback ( void * );
		void interrupt();
	};
	
#endif

/*
  Notes:
  
  This class supports an external interface and one for derived classes. The
  public interface is similar to the other command classes. An app initiates a
  task by creating an instance of a class derived from BreakCmd then calling its
  execute() function. This class defines an overloaded execute() allowing user
  specification of a callback function, which allows the application to be
  notified on task completion or interruption. The callback must be of type
  TaskDoneCallback, defined in BreakCmd.h as function taking a pointer to a
  BreakCmd object, a bool, and an untyped pointer.
  
  The protected portion provides the interface for derived classes, which must
  implement doCommand() declared as pure virtual by Cmd. This member function
  must set _done to TRUE on completion.
  
  cleanup() is used when the task ends; it can be overriden by derived classes.
  
  The derived class' doCommand() can use updateMessage() to inform the user of
  progress.
  
  updateTimeout can be used to alter the timeout for the pixmap progression
  during the progression itself.
*/
