/*
 MainWindow.cc - create application's top-level window

 Modified by Will 18, 2001, will@opaline.demon.co.uk
 Added protocol callback for WM_DELETE_WINDOW directed to empty method
 to force use of app-defined 'quit'.

 Adapted from
 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.
*/

#include "Application.h"
#include "MainWindow.h"

#include <Xm/MainW.h>
#include <assert.h>
#include <Xm/Protocols.h>


MainWindow::MainWindow ( const char *name ) : Interface ( name )
{
	_workArea = NULL;
	assert ( application ) ;	// App obj must exist prior to MainWindow obj
	application->registerWindow ( this );
}


MainWindow::~MainWindow()
{
	application->unregisterWindow ( this );	// unregister window with app obj
}


void MainWindow::initialize()
{
	Atom WM_DELETE_WINDOW;
	_w = XtCreatePopupShell ( _name, applicationShellWidgetClass,
				  application->baseWidget(), NULL, 0 );
							
	installDestroyHandler();

	_main = XtCreateManagedWidget ( "mainWindow", xmMainWindowWidgetClass,
					_w, NULL, 0 );

	_workArea = createWorkArea ( _main );
	assert ( _workArea );
	
	XtVaSetValues ( _main, XmNworkWindow, _workArea, NULL );

	// cater for WM 'X' pressed
	WM_DELETE_WINDOW = XmInternAtom ( XtDisplay ( _w ), "WM_DELETE_WINDOW", False );
	XmAddWMProtocols ( _w, &WM_DELETE_WINDOW, 1 );
	XmAddWMProtocolCallback ( _w, WM_DELETE_WINDOW, clientMessageProc, ( XtPointer ) this );

	if ( !XtIsManaged ( _workArea ) )
		XtManageChild ( _workArea );
}


void MainWindow::clientMessageProc ( Widget _w, XtPointer clientData, XtPointer )
{
	// empty - disallow WM_DELETE_WINDOW, force use of 'Exit'
}


void MainWindow::manage()
{
	assert ( _w );
	XtPopup ( _w, XtGrabNone );
	
	if ( XtIsRealized ( _w ) )	// map if iconified
		XMapRaised ( application->display(), XtWindow ( _w ) );
}


void MainWindow::unmanage()
{
	assert ( _w );
	XtPopdown ( _w );
}


void MainWindow::iconify()
{
	assert ( _w );
	// set widget to have initial iconic state if base widget unrealised
	XtVaSetValues ( _w, XmNiconic, TRUE, NULL );

	// if window realised, iconify it
	if ( XtIsRealized ( _w ) )
		XIconifyWindow ( application->display(), XtWindow ( _w ), 0 );
}
