// MenuWindowApp.cc

#include "MenuWindowApp.h"
#include <MenuBar.h>
#include "MenuApp.h"
#include <NOPCmd.h>
#include <UndoCmd.h>
#include <CmdList.h>
#include <Xm/DrawingA.h>


MenuWindowApp::MenuWindowApp ( const char *name ) : MenuWindow ( name )
{
	// create three NOPCmd objs
	_a = new NOPCmd ( "A" );
	_b = new NOPCmd ( "B" );
	_c = new NOPCmd ( "C", FALSE );
	
	_a->addToActivationList ( _b );
	_a->addToActivationList ( _c );
	_a->addToDeactivationList ( _a );
	
	_b->addToActivationList ( _a );
	_b->addToActivationList ( _c );
	_b->addToDeactivationList ( _b );
	
	_c->addToActivationList ( _a );
	_c->addToActivationList ( _b );
	_c->addToDeactivationList ( _c );
}

/*
  MainWindow derived classes must define this createWorkArea()
  This example xreates and returns an XmDrawingArea widget (unused, but every
  MainWindow object must have a work area).
*/

Widget MenuWindowApp::createWorkArea ( Widget parent )
{
	_canvas = XtCreateWidget ( "canvas", xmDrawingAreaWidgetClass,
							 	parent, NULL, 0 );
								
	return ( _canvas );
}

/*
  createMenuPanes() is called by MenuWindow after the window's MenuBar object
  has been created. This function adds menu panes to the MenuBar object by
  constructing a list of Cmd objects from which to create each menu pane.

  menuWindowApp creates three menu panes; the first is "Undo", provided by the
  backtrack object and also the "Quit", "Open" and "Icon" objects instantiated in
  the MenuApp constructor. The member function here adds each of these objects to
  a CmdList object and adds a menu pane named "Application" to the window's menu
  bar.
  
  Next a new CmdList object is created and a list built containing the "X",, "Y"
  and "Z" objects provided by the MenuApp object. Last, the third menu pane
  contains the "A", "B" and "C" command objects created by each instance of
  MenuWindowApp.
*/

void MenuWindowApp::createMenuPanes()
{	
	CmdList *cmdList;

	// create Application pane with app-wide commands
	cmdList = new CmdList ( "Application" );
	cmdList->add ( backtrack );	// undo last command
	cmdList->add ( menuApp->manageCmd() );
	cmdList->add ( menuApp->iconifyCmd() );
	cmdList->add ( menuApp->quitCmd() );
	_menuBar->addCommands ( cmdList );
	delete cmdList;
	
	// create menu pane of NOPCmd objs showing Cmd objs having multiple
	// interfaces
	cmdList = new CmdList ( "XYZ" );
	cmdList->add ( menuApp->xCmd() );
	cmdList->add (menuApp->yCmd() );
	cmdList->add ( menuApp->zCmd() );
	_menuBar->addCommands ( cmdList );
	delete cmdList;
	
	// create window-specific menu pane with commands independent within
	// each window
	cmdList = new CmdList ( "ABC" );
	cmdList->add ( _a );
	cmdList->add ( _b );
	cmdList->add ( _c );
	_menuBar->addCommands ( cmdList );
	delete cmdList;
}
	
